% Estimating MFPTs using walks of different length

load('networks.mat');
W = W_fb;  % 4039 nodes
Wt = W';  % for large networks, may want to keep a single instance of the transpose

%% Settings

n = size(W, 1);  % chain / network size
nidx = 5;  % number of target states to estimate MFPTs to / from
use_parallel = true;  % allow to use as many threads as the number of physical cores
nits = uint64([ 10^3, 10^4, 10^5, 10^6 ]);  % number of steps for the random walk

fprintf('* Chain with %d states.\n', n);
fprintf('* Estimating MFPTs to/from %d states.\n', nidx);
fprintf('* Numbers of iterations (random walk lengths):\n');
nits'

%% Selecting which nodes / states to estimate MFPTs to and from
idx = uint64(sort(uint64(randsample(n, nidx, false))));
fprintf('* Target states:\n');
idx'

%% estimating MFPTs
t_mfpt_est = tic;
M_ests = mfpt_est(Wt, idx, nits, use_parallel);
t_mfpt_est = toc(t_mfpt_est);
fprintf('* Obtained %d matrices of MFPT estimates in %fsec.\n', length(nits), t_mfpt_est);

%% visualizing

t_mfpt_exact = tic;
M_true = mfpt(W);
t_mfpt_exact = toc(t_mfpt_exact);
fprintf('* Computed exact MFPTs in %fsec.\n', t_mfpt_exact);

if t_mfpt_exact < t_mfpt_est
	fprintf([
		'* NOTE: The chain is so small that computing all MFPTs exactly (took %fsec)\n', ...
		'        is faster than estimating them with nit = %d (took %fsec).\n'
	], t_mfpt_exact, max(nits), t_mfpt_est);
else
	fprintf([
		'* NOTE: The chain is large enough so that estimating select MFPTs using\n', ...
		'        %d iterations (took %fsec.) is faster than computing all MFPTs\n', ...
		'        exactly (took %fsec).\n'
	], max(nits), t_mfpt_est, t_mfpt_exact);
end

figure;
mask = round(linspace(1, nnz(M_ests{1}), min(1000, nnz(M_ests{1}))));
colors = 1 - autumn(length(nits));
markers = { '*', 'x', 'o', 'd', '^', '+' };
legs = { };
for init = length(nits) : -1 : 1
	M_est = M_ests{init};

	[i_est, j_est, v_est] = find(M_est);
	if init == length(nits)
		v = M_true(sub2ind(size(M_est), i_est, j_est));
		[v, idx] = sort(v, 'ascend');
	end

	v_est = v_est(idx);
	plot(v_est(mask), '.', 'Marker', markers{mod(init - 1, length(markers)) + 1}, 'LineWidth', 1);
	hold on;
	legs{end + 1} = sprintf('$n_{it} = 10^{%.1f}$', log10(double(nits(init)))); %#ok<SAGROW>
end
plot(v(mask), '.', 'Color', [ 0, 0, 0 ], 'LineWidth', 1);
legs{end + 1} = 'Exact MFPTs';
hold off;
ylim([0, v(round(0.95 * length(v)))]);
legend(legs, 'Location', 'northwest', 'Interpreter', 'latex');
title('Exact MFPTs vs. Estimated of different quality', 'Interpreter', 'latex');
set(gca, 'FontSize', 14);