#ifndef __COMPACT_MAT_H__
#define __COMPACT_MAT_H__

#include <stdint.h>
#include <vector>
#include "mex_safe.h"
#include "debug.h"

#define SUB2IND(i, j, nrows) ((j) * (nrows) + (i))


class CompactMat {

private:
	
	uint64_t _n;
	uint64_t *_nzrc_idx;
	bool *_is_nzrc_ind;
	uint64_t _num_nzrc;
	uint64_t *_zrc_idx;
	uint64_t _num_zrc;
	uint64_t *_i2ii;
	double *_core;
	double *_fat;
	double *_tall;
	
	inline uint64_t sub2ind_core(uint64_t i, uint64_t j) const {
		ASSERT(_is_nzrc_ind[i], "sub2ind_core: i is not in _nzrc_idx");
		ASSERT(_is_nzrc_ind[j], "sub2ind_core: j is not in _nzrc_idx");
		uint64_t ind = SUB2IND(_i2ii[i], _i2ii[j], _num_nzrc);
		ASSERT((ind >= 0) && (ind < _num_nzrc * _num_nzrc), ("sub2ind_core: ind = " + std::to_string(ind) + " is out of range.").c_str());
		return ind;
	}

	inline uint64_t sub2ind_fat(uint64_t i, uint64_t j) const {
		ASSERT(_is_nzrc_ind[i], "sub2ind_fat: i is not in _nzrc_idx");
		ASSERT(!_is_nzrc_ind[j], "sub2ind_fat: j is in _nzrc_idx");
		uint64_t ind = SUB2IND(_i2ii[i], _i2ii[j], _num_nzrc);
		ASSERT((ind >= 0) && (ind < _num_nzrc * _n), ("sub2ind_fat: ind = " + std::to_string(ind) + " is out of range.").c_str());
		return ind;
	}

	inline uint64_t sub2ind_tall(uint64_t i, uint64_t j) const {
		ASSERT(!_is_nzrc_ind[i], "sub2ind_fat: i is in _nzrc_idx");
		ASSERT(_is_nzrc_ind[j], "sub2ind_fat: j is not in _nzrc_idx");
		uint64_t ind = SUB2IND(_i2ii[i], _i2ii[j], _num_zrc);
		ASSERT((ind >= 0) && (ind < _num_nzrc * _n), ("sub2ind_fat: ind = " + std::to_string(ind) + " is out of range.").c_str());
		return ind;
	}

public:

	CompactMat(uint64_t n, const uint64_t *nzrc_idx, uint64_t num_nzrc, double fill = 0.0);
	mxArray* to_sparse_mat() const;
#ifdef DEBUG
	void print_state() const;
#endif
	
	static CompactMat *merge(const std::vector<CompactMat*> cms);
	
	inline double get_ij(uint64_t i, uint64_t j) const {
		ASSERT((i >= 0) && (i < _n), ("get_ij: i = " + std::to_string(i) + " is out of range.\n").c_str());
		ASSERT((j >= 0) && (j < _n), ("get_ij: j = " + std::to_string(j) + " is out of range.\n").c_str());

		if (_is_nzrc_ind[i] && _is_nzrc_ind[j])
			return _core[sub2ind_core(i, j)];
		else if (_is_nzrc_ind[i] && !_is_nzrc_ind[j])
			return _fat[sub2ind_fat(i, j)];
		else if (!_is_nzrc_ind[i] && _is_nzrc_ind[j])
			return _tall[sub2ind_tall(i, j)];
		else {
			mexErrMsgTxt_safe(("get_ij: attempt to access the empty chunk with i = " + std::to_string(i) + " and j = " + std::to_string(j) + ".\n").c_str());
			return 0.0;
		}
	}
	
	inline void set_ij(uint64_t i, uint64_t j, double val) {
		ASSERT((i >= 0) && (i < _n), ("set_ij: i = " + std::to_string(i) + " is out of range.\n").c_str());
		ASSERT((j >= 0) && (j < _n), ("set_ij: j = " + std::to_string(j) + " is out of range.\n").c_str());

		if (_is_nzrc_ind[i] && _is_nzrc_ind[j])
			_core[sub2ind_core(i, j)] = val;
		else if (_is_nzrc_ind[i] && !_is_nzrc_ind[j])
			_fat[sub2ind_fat(i, j)] = val;
		else if (!_is_nzrc_ind[i] && _is_nzrc_ind[j])
			_tall[sub2ind_tall(i, j)] = val;
		else {
			mexErrMsgTxt_safe(("set_ij: attempt to access the empty chunk with i = " + std::to_string(i) + " and j = " + std::to_string(j) + ".\n").c_str());
			return;
		}
	}
};

#endif // __COMPACT_MAT_H__