#include <stdint.h>
#include <iostream>
#include <string>
#include <limits>
#include <cmath>
#include <vector>
#include <algorithm>
#include <thread>
#include <random>
#include "mex_safe.h"
#include "debug.h"
#include "compact_mat.h"


class HitCounter {
private:
	uint64_t _n;
	std::vector<uint64_t> _hits;
	double _w_sum;
public:
	HitCounter(uint64_t n) {
		ASSERT(n > 0, "HitCounter: Invalid n.");
		_n = n;
		_hits.resize(n, 1);
		_w_sum = n;
	}
	void hit(uint64_t i) {
		_w_sum -= 1.0 / _hits[i];
		_w_sum += 1.0 / ++_hits[i];
	}
	uint64_t rnd_favor_rare() {
		std::random_device rd;
		std::uniform_real_distribution<double> uni_die(0.0, 1.0);
		double die = uni_die(rd);
		double s = 0.0;
		for (uint64_t i = 0; i < _n; ++i) {
			s += (1.0 / _hits[i]) / _w_sum;
			if (s >= die)
				return i;
		}
		ASSERT(false, "HitCounter: Invalid random choice.");
		return -1;
	}
};


void mfpt_est_iterate_core(const uint64_t *wt_ir, const uint64_t *wt_jc, const double *wt_pr, uint64_t n,
		uint64_t nit, const uint64_t *idx, uint64_t nidx, const bool *is_in_idx, double pr_restart,
		CompactMat *&cm_msum, CompactMat *&cm_mnum) {

	CompactMat cm_pt(n, idx, nidx);
	cm_msum = new CompactMat(n, idx, nidx, std::numeric_limits<double>::epsilon());
	cm_mnum = new CompactMat(n, idx, nidx);

	// random
	std::random_device rd;
	std::mt19937_64 rng(rd());
	std::uniform_int_distribution<uint64_t> uni_i(0, n - 1);
	std::uniform_int_distribution<uint64_t> uni_ii(0, nidx - 1);
	std::uniform_real_distribution<double> uni_die(0.0, 1.0);

	// hit counter
	HitCounter hc(n);
	
	// starting at a uniformly randomly chosen target node j
	uint64_t jj = uni_ii(rd);
	uint64_t j = idx[jj];
	if (pr_restart > 0)
		hc.hit(j);
	
	for (uint64_t t = 0; t < nit; ++t) {
		
		if (pr_restart > 0 && uni_die(rd) <= pr_restart) {
			// restart
			j = hc.rnd_favor_rare();
		} else {
			// jumping to a uniformly randomly chosen out-neighbor of j
			double die = uni_die(rd);
			double cums = 0.0;
			for (uint64_t loc = wt_jc[j]; loc < wt_jc[j + 1]; ++loc) {
				cums += wt_pr[loc];
				if (cums >= die) {
					j = wt_ir[loc];
					break;
				}
			}
			if (cums < die)
				mexErrMsgTxt_safe(("mfpt_est_iterate: invalid state transition matrix; edge weights in the out-neighborhood of node " + std::to_string(j + 1) + " do not sum up to 1.0.\n").c_str());
		}
		
		if (pr_restart > 0)
			hc.hit(j);

		// checking possible first passages to node j
		//		from all nodes -- if j is a target node, or
		//		from only target nodes if j is a non-target node
		uint64_t k_max = is_in_idx[j] ? n : nidx;
		for (uint64_t k = 0; k < k_max; ++k) {
			uint64_t i = is_in_idx[j] ? k : idx[k];
			double ptij = cm_pt.get_ij(i, j);
			double ptji = cm_pt.get_ij(j, i);
			if (ptij == 0 && ptji == 0) {
				// we are not tracking passage from i to j; and we are not tracking passage from j to i
				cm_pt.set_ij(j, i, t);
			} else if (ptij > 0) {
				// we are tracking passage from i to j
				double fpt = t - ptij; // (one another) first passage time from i to j
				cm_msum->set_ij(i, j, cm_msum->get_ij(i, j) + fpt); // accumulating FPT sum
				cm_mnum->set_ij(i, j, cm_mnum->get_ij(i, j) + 1.0); // accumulating FPT counts
				cm_pt.set_ij(i, j, 0.0); // stop tracking passage from i to j
				cm_pt.set_ij(j, i, (double)t); // start tracking passage from j to i, indicating that we were at j at time t
			}
		}
	}
}


double get_rw_restart_prob(uint64_t n, uint64_t nit, uint64_t nthreads);


void mfpt_est_iterate(const uint64_t *wt_ir, const uint64_t *wt_jc, const double *wt_pr, uint64_t n, uint64_t nit,
		const uint64_t *idx, uint64_t nidx, const bool *is_in_idx, uint64_t nthreads, mxArray *&msum, mxArray *&mnum) {

	double pr_restart = get_rw_restart_prob(n, nit, nthreads);
	
	if (nthreads <= 1) {
		CompactMat *cm_msum, *cm_mnum;
		mfpt_est_iterate_core(wt_ir, wt_jc, wt_pr, n, nit, idx, nidx, is_in_idx, pr_restart, cm_msum, cm_mnum);
		msum = cm_msum->to_sparse_mat();
		mnum = cm_mnum->to_sparse_mat();
		delete cm_msum;
		delete cm_mnum;
	} else {
		std::vector<std::thread> workers;
		std::vector<CompactMat*> msums(nthreads);
		std::vector<CompactMat*> mnums(nthreads);

		uint64_t nit_per_thread = (uint64_t)ceil((double)nit / nthreads);
		
		for (uint64_t l = 0; l < nthreads; ++l) {
			workers.push_back(std::thread(
				mfpt_est_iterate_core,
				wt_ir, wt_jc, wt_pr, n, nit_per_thread, idx, nidx, is_in_idx, pr_restart,
				std::ref(msums[l]), std::ref(mnums[l])
			));
		}

		for (uint64_t l = 0; l < nthreads; workers[l++].join());

		CompactMat *msums_merged = CompactMat::merge(msums);
		CompactMat *mnums_merged = CompactMat::merge(mnums);

		for (uint64_t l = 0; l < nthreads; ++l) {
			delete msums[l];
			delete mnums[l];
		}

		msum = msums_merged->to_sparse_mat();
		mnum = mnums_merged->to_sparse_mat();
		delete msums_merged;
		delete mnums_merged;
	}
}


double get_rw_restart_prob(uint64_t n, uint64_t nit, uint64_t nthreads) {
	static const double max_pr_restart = 0.0001;
	const double want_nvisits_per_node = 0.001;
	double n_restarts = want_nvisits_per_node * n;
	double nit_per_thread = (double)nit / nthreads;
	double pr_restart = n_restarts / nit_per_thread;
	pr_restart = (pr_restart < 1) ? std::min(max_pr_restart, pr_restart) : 0.0;
	return pr_restart;
}