function [ Msum, Mnum ] = mfpt_est_iterate(Wt, nit, idx, use_parallel)
%MFPT_EST_ITERATE Performs several iterations of MFPT estimation algorithm.
%
% INPUT:
%
%	Wt -- sparse transpose of the state transition matrix
%	nit -- number of iterations to perform
%	idx -- indices of the target nodes
%
% OUTPUT:
%
%	Msum -- cumulative first passage times between target sources and destinations
%	Mnum -- numbers of first passages between target sources and destinations (Msum ./ Mnum ~ MFPTs)
%

%% Input validation

if(nargin < 3)
    error('Not enough arguments.');
end

if(~ismatrix(Wt) || min(size(Wt)) ~= max(size(Wt)))
   error('Wtrans is either not a matrix or non-square.');
end

if ~issparse(Wt)
	error('Wtrans is not sparse.');
end

n = size(Wt, 1);

if any(any(Wt < 0)) || any(any(Wt > 1.0)) || norm(full(sum(Wt, 1)) - ones(1, n)) > 1e-12
	error('Wtrans is not row-stochastic.');
end

nit = uint64(max(1, round(nit)));

if any(idx < 1) || any(idx > n)
	error('Some target node indices are out of range.');
end

nc = numcores();
if islogical(use_parallel) && use_parallel
	nthreads = nc;
elseif isnumeric(use_parallel)
	if use_parallel > 2 * nc
		warning('Attempting to use more threads than twice the number of physical cores.');
	end
	nthreads = uint64(max(1, round(use_parallel)));
else
	nthreads = uint64(1);
end

%% Iterating

[ Msum, Mnum ] = mfpt_est_iterate_mex(Wt, nit, idx - 1, nthreads);

end


function [ num ] = numcores()
%NUMCORES Returns the number of physical cores.

err = false;
f = [];
try
	f = feature('numCores');
catch
	err = true;
end

if err || isempty(f)
	warning('numcores: unable to get the number of physical cores.');
	num = 1;
	return;
end

num = uint64(max(1, round(double(f))));
	
end
