function [ M, pi ] = mfpt(W, pi, Wt, tol, maxit, maxtr)
%MFPT(W, pi, Wt, tol, maxit, maxtr) Computes exact mean first passage times between all states of a Markov chain using
% the fundamental matrix method of Kemeny and Snell [1].
%
% INPUT:
%
%	W -- n-by-n (row-stochastic, irreducible) state transition matrix of the Markov chain.
%	pi -- Perron vector of W (l1-normalized dominant left eigenvector of W); if not provided or empty, will be computed.
%	W_trans -- W's transpose; if not provided or empty, will be computed.
%	tol -- tolerance used by the eigensolver and row-stochasticity test; default is 1e-14.
%	maxit -- maximum number of iterations used by the eigensolver; default is 300.
%	maxtr -- maximum number of attempts to extract Perron vector from W with different
%		initial states (if no convergence at the first attempt); default is 5.
%
% OUTPUT:
%
%	M -- n-by-n dense matrix of mean first passage times between all states of the Markov chain.
%	pi -- eigenvector centralities of states (l1-normalized left dominant eigenvector of W)
%
% EXAMPLE:
%
% 	% state transition matrix W (in this case, a bi-directed chain graph on n nodes)
% 	n = 21;
% 	I = eye(n, n);
% 	Self = diag(rand(n, 1));
% 	Next = circshift(eye(n, n), -1, 1);
% 	Prev = circshift(eye(n, n), 1, 1);
% 	W = sparse(Self + (I - Self) / 2 * (Next + Prev));
% 
% 	plot(digraph(W));
% 	title('State transition matrix');
% 
% 	Wt = W';
% 
% 	[pi, ~] = eigs(Wt, 1, 'largestreal');
% 	if pi(1) < 0; pi = -pi; end
% 	pi = pi / sum(pi);
% 
% 	M1 = mfpt(W);  % basic computation
% 	M2 = mfpt(W, pi);  % computation with pre-computed Perron vector
% 	M3 = mfpt(W, nan, Wt);  % computation with providing W's transpose
% 	M4 = mfpt(W, nan, Wt, 1e-08 * n, 10 * n, 1);  % computation with providing Wt = W' and parameters for Perron vector computation
% 
% 	figure('Position', [10, 10, 1200, 200]);
% 	for i = 1 : 4
% 		subplot(1, 4, i);
% 		pcolor(eval(['M', num2str(i)]));
% 		colormap(1 - bone);
% 		colorbar();
% 		title(sprintf('MFPTs (Example %d)', i));
% 	end
%
% REFERENCES:
%
% [1] Kemeny, John, G. and J. Laurie Snell (1976) Finite Markov Chains. Springer-Verlag. Berlin

if nargin < 1
	error('State transition matrix W is not provided.');
end

n = size(W, 1);

if nargin < 4
	tol = 1e-10;
end

if ~ismatrix(W) || ~isreal(W) || (n ~= size(W, 2)) || norm(sum(W, 2) - ones(n, 1), 1) > tol
	error('Invalid state transition matrix W.');
end

if nargin < 2 || isempty(pi) || isscalar(pi) && isnan(pi)
	if nargin < 3 || isempty(Wt) || isscalar(Wt) && isnan(Wt)
		Wt = W';
	end
	if nargin < 5
		maxit = 300;
	end
	if nargin < 6
		maxtr = 5;
	end
	ops = struct('tol', tol, 'maxit', maxit, 'fail', 'keep');
	pi = perron_vec(Wt, maxtr, ops);
end

A = kron(ones(n, 1), pi');
I = eye(n);
Z = inv(I - W + A);
E = ones(n, n);
D = diag(1 ./ pi);
Zdg = diag(diag(Z));
M = (I - Z + E * Zdg) * D;

end


function [ pi ] = perron_vec(W_trans, maxtr, ops)
%PERRON_VEC Computes l1-normalized left dominant eigenvector of W (right dominant eigenvector of W_trans = W').

	for itr = 1 : maxtr
		err = false;
		try
			[pi, ~, no_conv] = eigs(W_trans, 1, 'largestreal', ...
				'Tolerance', ops.tol, 'MaxIterations', ops.maxit, 'FailureTreatment', ops.fail);
		catch
			err = true;
		end
		if ~err
			break;
		end
	end

	if err
		error('Unable to compute Perron vector of W -- eigensolver error.');
	end

	if no_conv
		warning('Perron vector of W did not converge; attempting to proceed.');
	end

	pi = abs(pi);
	pi = pi ./ sum(pi);

end