function [ M_est ] = mfpt_est(Wt, idx, nit, use_parallel, no_est_val)
%MFPT_EST Estimates mean first passage times (MFPTs) between a subset of states in a Markov chain
% using a finite-time random walk.
%
% INPUT:
%
%	Wt -- an n-by-n sparse transpose of the row-stochastic state transition matrix of the Markov chain.
%
%	idx -- (1-based) indices of the target states to estimate MFPTs to/from.
%
%	nit -- total number of steps in the random walk over the chain. If a vector is provided instead
%		of a scalar, the function will produce an cell array of MFPT estimates with corresponding nit values.
%
%	use_parallel -- either a logical indicating whether using multiple threads is allowed (default
%		is true, corresponding to using as many threads as the number of physical cores), or
%		an integer indicating the number of threads to use. Default is true.
%
%	no_est_val -- a scalar that will be used to denote MFPTs for which we do not have any estimate.
%		Default is nan.
%
% OUPUT:
%
%	M_est -- an n-by-n sparse matrix of estimates of the mean first passage time between states idx.
%		MFPTs between states not included in idx are not stored in M_est, even though accesing them
%		through M_est would produce zero values. If nit was an array rather than a scalar, then M_est
%		is a cell-array of MFPT estimates, each corresponding to different nit(i).
%
% NOTE:
%
%	For MFPT estimation, we need only the transpose Wt of the state transition matrix W. Thus, if the
%	original W is not used anywhere else, it may make sense to store only Wt, particularly, when the
%	chain is very large.
%
% EXAMPLE:
%
% 	use_parallel = true;  % allow to use as many threads as the number of physical cores
% 
% 	% Creating the state transition matrix W. Note: for MFPT estimation, we need only its transpose.
% 	n = 21;
% 	I = eye(n, n);
% 	Self = diag(rand(n, 1));
% 	Next = circshift(eye(n, n), -1, 1);
% 	Prev = circshift(eye(n, n), 1, 1);
% 	W = sparse(Self + (I - Self) / 2 * (Next + Prev));
% 	Wt = W';  % for large networks, may want to keep a single instance of the transpose
% 
% 	nidx = 5;
% 	idx = uint64(sort(uint64(randsample(n, nidx, false)))); % selecting which nodes / states to estimate MFPTs to and from
% 	
% 	nit = 100000; % number of iterations to perform
% 
% 	M_est = mfpt_est(Wt, idx, nit, use_parallel); % estimating MFPTs
%

if nargin < 4
	use_parallel = true;
end

if nargin < 5
	no_est_val = nan;
end

ensure_mex_exists('mfpt_est_iterate_mex');

n = size(Wt, 1);
nit = unique(nit);
nits = [ nit(1), nit(2 : end) - nit(1 : end - 1) ];
M_est = cell(length(nits), 1);
for i = 1 : length(nit)
	cur_nit = nits(i);
	[ Msum, Mnum ] = mfpt_est_iterate(Wt, cur_nit, idx, use_parallel);
	
	if i == 1
		[tis, tjs] = find(Msum);
		tlind = sub2ind(size(Msum), tis, tjs);
		nzmax = length(tlind);
	end

	msum_v = Msum(tlind);
	mnum_v = Mnum(tlind);
	
	if i > 1
		msum_v = msum_v + msum_v_prev;
		mnum_v = mnum_v + mnum_v_prev;
	end

	no_est_indic = msum_v < 0.5;
	rat = msum_v ./ mnum_v;
	rat(no_est_indic) = no_est_val;
	M_est{i} = sparse(tis, tjs, rat, n, n, nzmax);
	
	msum_v_prev = msum_v;
	mnum_v_prev = mnum_v;
end

if length(M_est) == 1
	M_est = M_est{1};
end

end


function ensure_mex_exists(mex_fname)

if nargin < 1
	error('mex_fname not provided.');
end

if exist(mex_fname, 'file') == 3
	return;
end

mex_make_fname = [ mex_fname, '_make' ];

compiling = false;
if exist(mex_make_fname, 'file') == 2
	fprintf(2, [ mex_fname, '.mex* file was not found; trying to compile it.\n' ]);
	compiling = true;
	eval(mex_make_fname);
end

if exist(mex_fname, 'file') ~= 3
	error([ 'Accompanying ', mex_fname, '.mex* file is not found, ', ...
		'and its compilation attempt has failed. ' ]);
elseif compiling
	fprintf(2, [ mex_fname, '.mex* has been successfully compiled.\n' ]);
end

end
